import {
  Body,
  Controller,
  Get,
  Post,
  Put,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ApiBearerAuth, ApiBody, ApiOperation, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionInfraccionService } from './institucion-infraccion.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { CreateInstitucionInfraccionDto } from './dto/input/create.dto';
import { UpdateInstitucionInfraccionDto } from './dto/input/update.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { InfraccionOutputDto } from './dto/output/infraccion.dto';
import { InstitucionInfraccionOutputDto } from './dto/output/institucion-infraccion.dto';

@Controller('institucion-infraccion')
@ApiTags('institucion-infraccion')
export class InstitucionInfraccionController {
  constructor(
    private institucionInfraccionService: InstitucionInfraccionService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(MessageOutputDto)
  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Crear una nueva infracción.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: { ejemplo: { value: { infraccion: '' } } },
  })
  create(@Request() req, @Body() body: CreateInstitucionInfraccionDto) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdmin(superAdmin);
    return this.institucionInfraccionService.create(body.infraccion);
  }

  @Serealize(InfraccionOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Todas las infracciones.' })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const superAdmin: Operador = req.user.operador;

    this.validarUsuarioService.validarSuperAdmin(superAdmin);
    return this.institucionInfraccionService.findAll();
  }

  @Serealize(InstitucionInfraccionOutputDto)
  @Get('infracciones')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Todas las instituciones infracción de un operador.',
  })
  @ApiBearerAuth('jwt')
  infracciones(@Request() req) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.institucionInfraccionService.findAllOperador(operador);
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Actualizar la información de una institución infracción.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: {
      ejemplo: { value: { id_institucion_infraccion: 603, dias_multa: 7 } },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionInfraccionDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.institucionInfraccionService.update(admin, body);
  }
}
