import {
  Body,
  Controller,
  Get,
  Post,
  Put,
  Query,
  Request,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import {
  ApiBearerAuth,
  ApiBody,
  ApiOperation,
  ApiQuery,
  ApiTags,
} from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { MultaService } from './multa.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { IdEquipoPaginaDto } from '../dto/input/id-equipo-pagina.dto';
import { IdUsuarioPaginaDto } from '../dto/input/id-usuario-pagina.dto';
import { MultarDto } from './dto/input/multar.dto';
import { MultasDto } from './dto/input/multas.dto';
import { QuitarMultarDto } from './dto/input/quitar.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { MultasOutputDto } from './dto/output/multas.dto';
import { MultasEquipoOutputDto } from './dto/output/multas-equipo.dto';
import { MultasMinOutputDto } from './dto/output/multas-min.dto';
import { MultasUsuarioOutputDto } from './dto/output/multas-usuario.dto';

@Controller('multa')
@ApiTags('multa')
export class MultaController {
  constructor(
    private multaService: MultaService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(MessageOutputDto)
  @Post()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Multar usuario.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: {
      ejemplo: {
        value: {
          descripcion: '',
          id_institucion_infraccion: 1,
          id_prestamo: 1,
        },
      },
    },
  })
  multar(@Request() req, @Body() body: MultarDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.multaService.multaExtemporanea(
      operador,
      body.id_prestamo,
      body.descripcion,
      body.id_institucion_infraccion,
    );
  }

  @Serealize(MultasOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Información de 25 multas dependiendo de los filtros enviados.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Nombre que se quiere usar como filtro.',
    name: 'nombre',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Usuario que se quiere usar como filtro.',
    name: 'usuario',
    type: 'string',
    required: false,
  })
  multas(@Request() req, @Query() query: MultasDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.multaService.findAll(operador, query);
  }

  @Serealize(MultasMinOutputDto)
  @Get('multas-usuario-activas')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Todas las multas activas de un usuario.' })
  @ApiBearerAuth('jwt')
  multasActivas(@Request() req) {
    const usuario: Usuario = req.user.usuario;

    this.validarUsuarioService.validarUsuario(usuario);
    return this.multaService.findMultasActivasByUsuario(usuario);
  }

  @Serealize(MultasEquipoOutputDto)
  @Get('multas-equipo')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Información de 25 multas de un equipo dependiendo de los filtros enviados.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id del equipo.',
    name: 'id_equipo',
    type: 'string',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario',
    name: 'pagina',
    type: 'string',
  })
  multasEquipo(@Request() req, @Query() query: IdEquipoPaginaDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.multaService.findAllByIdEquipo(
      operador,
      parseInt(query.id_equipo),
      parseInt(query.pagina),
    );
  }

  @Serealize(MultasUsuarioOutputDto)
  @Get('multas-usuario')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Información de 25 multas de un usuario dependiendo de los filtros enviados.',
  })
  @ApiBearerAuth('jwt')
  @ApiQuery({
    description: 'Id del usuario.',
    name: 'id_usuario',
    type: 'string',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario',
    name: 'pagina',
    type: 'string',
  })
  multasUsuario(@Request() req, @Query() query: IdUsuarioPaginaDto) {
    const operador: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(operador);
    return this.multaService.findAllByIdUsuario(
      operador,
      parseInt(query.id_usuario),
      parseInt(query.pagina),
    );
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({ description: 'Variables que necesita el endpoint.' })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: { example: { value: { id_institucion_usuario: 1 } } },
  })
  quitarMulta(@Request() req, @Body() body: QuitarMultarDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.multaService.quitarMulta(admin, body.id_institucion_usuario);
  }

  // @Get('reporte')
  // @UseGuards(AuthGuard('jwt'))
  // @ApiBearerAuth('jwt')
  // reporte(@Request() req) {
  //   const operador: Operador = req.user.operador;

  //   this.validarUsuarioService.validarAdminOperador(operador);
  //   // return this.multaService.findAll(operador, query);
  // }
}
