import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Between, FindOptionsWhere, Like, MoreThan, Repository } from 'typeorm';
import { Institucion } from '../institucion/entity/institucion.entity';
import { Operador } from './entity/operador.entity';
import { FullInformacionOperadorView } from './entity/views/full-informacion-operador.view';
import { BcryptService } from '../bcrypt/bcrypt.service';
import { InstitucionService } from '../institucion/institucion.service';
import { NodemailerService } from '../nodemailer/nodemailer.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';

@Injectable()
export class OperadorService {
  constructor(
    @InjectRepository(Operador) private repository: Repository<Operador>,
    @InjectRepository(FullInformacionOperadorView)
    private fullInformacionOperadorView: Repository<FullInformacionOperadorView>,
    private bcryptService: BcryptService,
    private institucionService: InstitucionService,
    private nodemailerService: NodemailerService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  async create(
    id_institucion: number | Institucion,
    id_tipo_usuario: number,
    operador: string,
    nombre: string,
    correo: string,
    password?: string,
  ): Promise<{ message: string }> {
    const institucion =
      typeof id_institucion === 'number'
        ? await this.institucionService.findById(id_institucion)
        : id_institucion;
    const tipoUsuario = await this.tipoUsuarioService.findById(id_tipo_usuario);

    // Buscamos un registro de admin con ese nombre
    return this.findAdmin(operador)
      .then((existeAdmin) => {
        // Sacamos error si existe
        if (existeAdmin)
          throw new ConflictException(
            'Ya existe un admin con ese nombre, intenta de nuevo con otro.',
          );
        // Buscamos un registro de operador con ese nombre
        return this.findOperador(institucion, operador);
      })
      .then(async (existeOperador) => {
        // Sacamos error si existe
        if (existeOperador)
          throw new ConflictException(
            'Ya existe un operador en esta institución con ese nombre, intenta de nuevo con otro.',
          );
        // Generamos un password por si no se mando uno
        if (!password) password = this.bcryptService.generarPassword();
        // Creamos y guardamos un registro
        return this.repository.save(
          this.repository.create({
            correo,
            nombre,
            operador,
            password: this.bcryptService.encriptar(password),
            institucion,
            tipoUsuario,
          }),
        );
      })
      .then((operador) =>
        // Enviamos un correo con credenciales
        this.nodemailerService.sendEmail({
          email: operador.correo,
          subject: 'Credenciales Pc Puma',
          html:
            tipoUsuario.id_tipo_usuario === 3
              ? this.nodemailerService.correoPasswordAdmin(
                  operador.operador,
                  password,
                )
              : this.nodemailerService.correoPasswordOperador(
                  operador.operador,
                  password,
                ),
        }),
      )
      .then((_) => ({
        message: `Se creó correctamente un nuevo ${tipoUsuario.tipo_usuario.toLowerCase()}.`,
      }));
  }

  findAdmin(admin: string): Promise<Operador> {
    return this.repository
      .findOne({
        where: {
          operador: admin,
          tipoUsuario: { id_tipo_usuario: Between(2, 3) },
        },
      })
      .then((admin) => this.llenarIds(admin));
  }

  async findAll(
    admin: Operador,
    filtros: {
      pagina: string;
      id_institucion?: string;
      id_tipo_usuario?: string;
      operador?: string;
    },
  ): Promise<[Operador[], number]> {
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(parseInt(filtros.id_institucion))
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? await this.tipoUsuarioService.findById(
          parseInt(filtros.id_tipo_usuario),
        )
      : null;
    const busqueda: FindOptionsWhere<FullInformacionOperadorView> = {
      id_tipo_usuario: MoreThan(2),
    };

    if (admin.tipoUsuario.id_tipo_usuario > 2)
      busqueda.id_institucion = admin.institucion.id_institucion;
    else if (institucion) busqueda.id_institucion = institucion.id_institucion;
    if (filtros.operador) busqueda.operador = Like(`${filtros.operador}%`);
    if (tipoUsuario) busqueda.id_tipo_usuario = tipoUsuario.id_tipo_usuario;
    return this.fullInformacionOperadorView
      .findAndCount({
        where: busqueda,
        take: 25,
        skip: (parseInt(filtros.pagina) - 1) * 25,
      })
      .then((infoOperadores) => {
        const operadores: Operador[] = [];

        for (let i = 0; i < infoOperadores[0].length; i++)
          operadores.push(
            this.repository.create({
              id_operador: infoOperadores[0][i].id_operador,
              activo: infoOperadores[0][i].activo === 1,
              correo: infoOperadores[0][i].correo,
              nombre: infoOperadores[0][i].nombre,
              operador: infoOperadores[0][i].operador,
              institucion: {
                id_institucion: infoOperadores[0][i].id_operador,
                institucion: infoOperadores[0][i].institucion,
              },
              tipoUsuario: {
                id_tipo_usuario: infoOperadores[0][i].id_tipo_usuario,
                tipo_usuario: infoOperadores[0][i].tipo_usuario,
              },
            }),
          );
        return [operadores, infoOperadores[1]];
      });
  }

  findAllAdmin(admin: Operador): Promise<Operador[]> {
    return this.repository.find({
      select: ['id_operador', 'operador'],
      where: { institucion: admin.institucion },
    });
  }

  findById(id_operador: number): Promise<Operador> {
    return this.repository
      .findOne({
        select: [
          'id_operador',
          'activo',
          'correo',
          'id_institucion',
          'id_tipo_usuario',
        ],
        where: { id_operador },
      })
      .then((operador) => {
        if (!operador)
          throw new NotFoundException('No existe este id operador.');
        return this.llenarIds(operador);
      });
  }

  findOperador(institucion: Institucion, operador: string): Promise<Operador> {
    return this.repository
      .findOne({
        where: { operador, institucion, tipoUsuario: { id_tipo_usuario: 4 } },
      })
      .then((operador) => this.llenarIds(operador));
  }

  llenarIds(operador: Operador): Operador {
    if (operador) {
      operador.tipoUsuario = this.tipoUsuarioService.crearTipoUsuario(
        operador.id_tipo_usuario,
      );
      if (operador.id_institucion)
        operador.institucion = this.institucionService.crearInstitucion(
          operador.id_institucion,
        );
    }
    return operador;
  }

  passwordReset(
    admin: Operador,
    id_operador: number,
    password?: string,
  ): Promise<{ message: string }> {
    return this.findById(id_operador)
      .then((operador) => {
        this.validarUpdate(admin, operador);
        // Generamos un password por si no se mando uno
        if (!password) password = this.bcryptService.generarPassword();
        // Encriptamos password
        operador.password = this.bcryptService.encriptar(password);
        // Guardamos cambios
        return this.repository.save(operador);
      })
      .then((operador) =>
        // Enviamos un correo con credenciales
        this.nodemailerService.sendEmail({
          email: operador.correo,
          subject: 'Credenciales PC Puma',
          html:
            operador.tipoUsuario.id_tipo_usuario === 3
              ? this.nodemailerService.correoPasswordAdmin(
                  operador.operador,
                  password,
                )
              : this.nodemailerService.correoPasswordOperador(
                  operador.operador,
                  password,
                ),
        }),
      )
      .then((_) => ({ message: 'Se cambió correctamente la constraseña.' }));
  }

  sistema(): Operador {
    return this.repository.create({ id_operador: 1 });
  }

  update(
    admin: Operador,
    attrs: Partial<Operador>,
  ): Promise<{ message: string }> {
    return this.findById(attrs.id_operador)
      .then((operador) => {
        this.validarUpdate(admin, operador);
        // Asignamos valores enviados al objeto
        Object.assign(operador, attrs);
        // Guardamos cambios
        return this.repository.save(operador);
      })
      .then((_) => ({ message: 'Se guardaron los cambios correctamente.' }));
  }

  validarUpdate(admin: Operador, operador: Operador): void {
    // Validamos que el super admin solo puedar modificar admins
    if (
      admin.tipoUsuario.id_tipo_usuario === 2 &&
      operador.tipoUsuario.id_tipo_usuario != 3
    )
      throw new ConflictException(
        `El super admin solo puede modificar la infomración de los admins.`,
      );
    if (admin.tipoUsuario.id_tipo_usuario === 3) {
      // Validamos que el admin no pueda modificar otro admin
      if (operador.tipoUsuario.id_tipo_usuario === 3)
        throw new ConflictException(
          `No puedes actualizar la información de otro admin.`,
        );
      // Validamos que el admin solo pueda modificar operadores de su institución
      if (
        admin.institucion.id_institucion != operador.institucion.id_institucion
      )
        throw new ConflictException(
          `No puedes actualizar la información de este operador porque no pertenece a tu institución.`,
        );
    }
  }
}
